package com.jiayue.ipp.idp.service.impl.an;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jiayue.ipp.common.data.entity.WindTurbineInfo;

import com.jiayue.ipp.common.data.entity.WindTurbineStatusData;
import com.jiayue.ipp.common.data.entity.an.ParsingFjInfo;
import com.jiayue.ipp.common.data.entity.an.ParsingInfo;
import com.jiayue.ipp.common.data.entity.an.ParsingUrl;
import com.jiayue.ipp.common.data.enums.FileTypeEnum;
import com.jiayue.ipp.idp.dto.FileAnalysisStatusDto;
import com.jiayue.ipp.idp.dto.ParsingConstant;
import com.jiayue.ipp.idp.mapper.an.ParsingFjInfoMapper;
import com.jiayue.ipp.idp.service.WindTurbineInfoService;
import com.jiayue.ipp.idp.service.WindTurbineStatusDataService;
import com.jiayue.ipp.idp.service.an.ParsingFjInfoService;
import com.jiayue.ipp.idp.service.an.ParsingInterface;
import com.jiayue.ipp.idp.util.FieldUtil;
import com.jiayue.ipp.idp.util.FileUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 风机解析业务层实现类
 *
 * @author tl
 * @date 2022-05-11 18:07:03
 */
@Service
@AllArgsConstructor
@Slf4j
@Transactional
public class ParsingFjInfoServiceImpl extends ServiceImpl<ParsingFjInfoMapper, ParsingFjInfo> implements ParsingFjInfoService, ParsingInterface {


    private final Long intervalTime = 300000l;

    private final WindTurbineStatusDataService windTurbineStatusDataService;

    private final WindTurbineInfoService windTurbineInfoService;


    @Override
    public boolean save(ParsingInfo parsingInfo) {
        return save((ParsingFjInfo) parsingInfo);
    }

    @Override
    public boolean deleteById(String id) {
        return removeById(id);
    }

    @Override
    public FileAnalysisStatusDto parsing(File file, ParsingUrl ftpParsingUrl) {
        FileAnalysisStatusDto fileAnalysisStatusDto = new FileAnalysisStatusDto();
        fileAnalysisStatusDto.setFileName(file.getName());
        fileAnalysisStatusDto.setFileType(FileTypeEnum.FJ.getCode()+"");

        try {
            List<ParsingFjInfo> parsingFjInfos = baseMapper.selectList(Wrappers.lambdaQuery(ParsingFjInfo.class).like(ParsingInfo::getStationCode, ftpParsingUrl.getStationCode()));
            if (parsingFjInfos.size() > 0) {
                for (ParsingFjInfo parsingFjInfo : parsingFjInfos) {
                    //判断文件名是否符合配置
                    if (file.getName().indexOf(parsingFjInfo.getFileName()) >= 0) {
                        List<WindTurbineStatusData> windTurbineStatusDataList = new ArrayList<>();
                        if (parsingFjInfo.getDataType().equals(FieldUtil.SINGLE)) {
                            List<WindTurbineInfo> windTurbineInfoList = windTurbineInfoService.list(Wrappers.lambdaQuery(WindTurbineInfo.class).eq(WindTurbineInfo::getStationCode, ftpParsingUrl.getStationCode()));
                            List<String> fileContent = FileUtil.getFileContent(file);
                            // 对#号开头的数据行获取
                            for (String str: fileContent){
                                if (str.startsWith("#")){
                                    // 循环获取每行数据风机名称
                                    String fjName = parsingName(parsingFjInfo.getEquipmentName(), str);
                                    List<WindTurbineInfo> filterWindTurbineInfos = windTurbineInfoList.stream().filter(i -> i.getName().equals(fjName)).collect(Collectors.toList());
                                    if (filterWindTurbineInfos.size() > 0) {
                                        WindTurbineStatusData windTurbineStatusData = new WindTurbineStatusData();
                                        windTurbineStatusData.setEquipmentId(filterWindTurbineInfos.get(0).getEquipmentNo());
                                        Long time = parsingTime(parsingFjInfo.getTime(), fileContent);
                                        if (time != 0l) {
                                            windTurbineStatusData.setTime(new Date(time));
                                            windTurbineStatusData.setActivePower(parsingRowValue(parsingFjInfo.getActivePower(), str));
                                            windTurbineStatusData.setCumulativeGeneratedEnergy(parsingRowValue(parsingFjInfo.getCumulativeGeneratedEnergy(), str));
                                            windTurbineStatusData.setDayGridConnectedHours(parsingRowValue(parsingFjInfo.getDayGridConnectedHours(), str));
                                            windTurbineStatusData.setWindWheelRatedSpeed(parsingRowValue(parsingFjInfo.getWindWheelRatedSpeed(), str));
                                            windTurbineStatusData.setDayElectricQuantity(parsingRowValue(parsingFjInfo.getDayElectricQuantity(), str));
                                            windTurbineStatusData.setElectricalCurrent(parsingRowValue(parsingFjInfo.getElectricalCurrent(), str));
                                            windTurbineStatusData.setPitchAngle(parsingRowValue(parsingFjInfo.getPitchAngle(), str));
                                            windTurbineStatusData.setPowerFactor(parsingRowValue(parsingFjInfo.getPowerFactor(), str));
                                            windTurbineStatusData.setReactivePower(parsingRowValue(parsingFjInfo.getReactivePower(), str));
                                            windTurbineStatusData.setVoltage(parsingRowValue(parsingFjInfo.getVoltage(), str));
                                            windTurbineStatusData.setWd(parsingRowValue(parsingFjInfo.getWd(), str));
                                            windTurbineStatusData.setWs(parsingRowValue(parsingFjInfo.getWs(), str));
                                            windTurbineStatusData.setT(parsingRowValue(parsingFjInfo.getT(), str));
                                            windTurbineStatusData.setStatus(parsingRowStatus(parsingFjInfo.getStatus(), str));
                                            windTurbineStatusData.setStationCode(ftpParsingUrl.getStationCode());
                                        } else {
                                            fileAnalysisStatusDto.setStatus("0");
                                            fileAnalysisStatusDto.setMessage("解析风机文件时间错误");
                                            log.error("解析FJ文件时间错误");
                                        }
                                        windTurbineStatusDataList.add(windTurbineStatusData);
                                    } else {
                                        fileAnalysisStatusDto.setStatus("0");
                                        fileAnalysisStatusDto.setMessage("风机缺少配置无法解析");
                                        log.info("风机 {} 缺少配置无法解析", fjName);
                                    }
                                }
                            }
                        } else {
                            fileAnalysisStatusDto.setStatus("0");
                            fileAnalysisStatusDto.setMessage("风机暂无多行解析方式");
                            log.info("fj暂无多行解析方式");
                        }
                        if (!"0".equals(fileAnalysisStatusDto.getStatus()) && windTurbineStatusDataList.size() > 0){
                            windTurbineStatusDataService.saveBatch(windTurbineStatusDataList);
                            log.info("解析FJ文件：" + file.getName() + " 成功！ O(∩_∩)O");
                            fileAnalysisStatusDto.setStatus("1");
                        }
                    }
                }
            }
        } catch (Exception e) {
            fileAnalysisStatusDto.setStatus("0");
            fileAnalysisStatusDto.setMessage("解析风机数据错误！");
            log.error("解析风机数据错误",e);
        } finally {
            return fileAnalysisStatusDto;
        }
    }
}
