package com.jiayue.ipp.idp.service.impl.an;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jiayue.ipp.common.data.entity.InverterInfo;
import com.jiayue.ipp.common.data.entity.InverterStatusData;
import com.jiayue.ipp.common.data.entity.an.ParsingInfo;
import com.jiayue.ipp.common.data.entity.an.ParsingNbqInfo;
import com.jiayue.ipp.common.data.entity.an.ParsingUrl;
import com.jiayue.ipp.common.data.enums.FileTypeEnum;
import com.jiayue.ipp.idp.dto.FileAnalysisStatusDto;
import com.jiayue.ipp.idp.mapper.an.ParsingNbqInfoMapper;
import com.jiayue.ipp.idp.service.InverterInfoService;
import com.jiayue.ipp.idp.service.InverterStatusDataService;
import com.jiayue.ipp.idp.service.an.ParsingInterface;
import com.jiayue.ipp.idp.service.an.ParsingNbqInfoService;
import com.jiayue.ipp.idp.util.FieldUtil;
import com.jiayue.ipp.idp.util.FileUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 逆变器解析业务层实现类
 *
 * @author tl
 * @date 2022-05-11 18:07:03
 */
@Service
@AllArgsConstructor
@Slf4j
@Transactional
public class ParsingNbqInfoServiceImpl extends ServiceImpl<ParsingNbqInfoMapper, ParsingNbqInfo> implements ParsingNbqInfoService, ParsingInterface {

    private final InverterStatusDataService inverterStatusDataService;

    private final InverterInfoService inverterInfoService;

    @Override
    public boolean save(ParsingInfo parsingInfo) {
        return save((ParsingNbqInfo) parsingInfo);
    }

    @Override
    public boolean deleteById(String id) {
        return removeById(id);
    }

    @Override
    public FileAnalysisStatusDto parsing(File file, ParsingUrl ftpParsingUrl) {
        FileAnalysisStatusDto fileAnalysisStatusDto = new FileAnalysisStatusDto();
        fileAnalysisStatusDto.setFileName(file.getName());
        fileAnalysisStatusDto.setFileType(FileTypeEnum.NBQ.getCode()+"");

        try {
            //查询符合该场站的nbq配置信息
            List<ParsingNbqInfo> parsingNbqInfos = baseMapper.selectList(Wrappers.lambdaQuery(ParsingNbqInfo.class).like(ParsingInfo::getStationCode, ftpParsingUrl.getStationCode()));
            if (parsingNbqInfos.size() > 0) {
                for (ParsingNbqInfo parsingNbqInfo : parsingNbqInfos) {
                    //判断文件名是否符合配置
                    if (file.getName().indexOf(parsingNbqInfo.getFileName()) >= 0) {
                        List<InverterStatusData> inverterStatusDataList = new ArrayList<>();
                        if (parsingNbqInfo.getDataType().equals(FieldUtil.SINGLE)) {
                            //查询对应该场站路径下的逆变器信息
                            List<InverterInfo> inverterInfoList = inverterInfoService.list(Wrappers.lambdaQuery(InverterInfo.class).eq(InverterInfo::getStationCode, ftpParsingUrl.getStationCode()));
                            //解析文件内容以行为单位的 List<String>
                            List<String> fileContent = FileUtil.getFileContent(file);
                            // 对#号开头的数据行获取
                            for (String str: fileContent){
                                if (str.startsWith("#")){
                                    String nbqName = parsingName(parsingNbqInfo.getEquipmentName(), str);
                                    List<InverterInfo> filterInverterInfos = inverterInfoList.stream().filter(i -> i.getName().equals(nbqName)).collect(Collectors.toList());
                                    if (filterInverterInfos.size() > 0) {
                                        InverterStatusData inverterStatusData = new InverterStatusData();
                                        inverterStatusData.setEquipmentId(filterInverterInfos.get(0).getEquipmentNo());
                                        Long time = parsingTime(parsingNbqInfo.getTime(), fileContent);
                                        if (time != 0l) {
                                            inverterStatusData.setActivePower(parsingRowValue(parsingNbqInfo.getActivePower(), str));
                                            inverterStatusData.setCumulativeGeneratedEnergy(parsingRowValue(parsingNbqInfo.getCumulativeGeneratedEnergy(), str));
                                            inverterStatusData.setDayGridConnectedHours(parsingRowValue(parsingNbqInfo.getDayGridConnectedHours(), str));
                                            inverterStatusData.setDayElectricQuantity(parsingRowValue(parsingNbqInfo.getDayElectricQuantity(), str));
                                            inverterStatusData.setElectricalCurrent(parsingRowValue(parsingNbqInfo.getElectricalCurrent(), str));
                                            inverterStatusData.setReactivePower(parsingRowValue(parsingNbqInfo.getReactivePower(), str));
                                            inverterStatusData.setPowerFactor(parsingRowValue(parsingNbqInfo.getPowerFactor(), str));
                                            inverterStatusData.setVoltage(parsingRowValue(parsingNbqInfo.getVoltage(), str));
                                            inverterStatusData.setStatus(parsingRowStatus(parsingNbqInfo.getStatus(), str));
                                            inverterStatusData.setStationCode(ftpParsingUrl.getStationCode());
                                        } else {
                                            fileAnalysisStatusDto.setStatus("0");
                                            fileAnalysisStatusDto.setMessage("解析逆变器文件时间错误");
                                            log.error("解析NBQ文件时间错误");
                                        }
                                        inverterStatusDataList.add(inverterStatusData);
                                    }
                                    else {
                                        fileAnalysisStatusDto.setStatus("0");
                                        fileAnalysisStatusDto.setMessage("逆变器缺少配置无法解析");
                                        log.info("逆变器 {} 缺少配置无法解析", nbqName);
                                    }
                                }
                            }
                        }
                        else {
                            fileAnalysisStatusDto.setStatus("0");
                            fileAnalysisStatusDto.setMessage("逆变器暂无多行解析方式");
                            log.info("nbq暂无多行解析方式");
                        }
                        if (!"0".equals(fileAnalysisStatusDto.getStatus()) && inverterStatusDataList.size() > 0){
                            inverterStatusDataService.saveBatch(inverterStatusDataList);
                            log.info("解析NBQ文件：" + file.getName() + " 成功！ O(∩_∩)O");
                            fileAnalysisStatusDto.setStatus("1");
                        }
                    }
                }
            }
        } catch (Exception e) {
            fileAnalysisStatusDto.setStatus("0");
            fileAnalysisStatusDto.setMessage("解析逆变器数据错误！");
            log.error("解析逆变器数据错误！", e);
        } finally {
            return fileAnalysisStatusDto;
        }
    }
}
